use super::Modelable;
use sqlite::Bindable;

impl Modelable for i32 {
    fn bind_to(&self, stmt: &mut sqlite::Statement, col: usize) -> sqlite::Result<()> {
        (*self as i64).bind(stmt, col)
    }
    fn build_from(stmt: &sqlite::Statement, col_offset: usize) -> sqlite::Result<(Self, usize)> where Self: Sized {
        stmt.read::<i64>(col_offset).map(|x| (x as i32, 1))
    }
}

impl Modelable for i64 {
    fn bind_to(&self, stmt: &mut sqlite::Statement, col: usize) -> sqlite::Result<()> {
        self.bind(stmt, col)
    }
    fn build_from(stmt: &sqlite::Statement, col_offset: usize) -> sqlite::Result<(Self, usize)> where Self: Sized {
        stmt.read(col_offset).map(|x| (x, 1))
    }
}

impl<'a> Modelable for &'a str {
    fn bind_to(&self, stmt: &mut sqlite::Statement, col: usize) -> sqlite::Result<()> {
        self.bind(stmt, col)
    }
    fn build_from(_stmt: &sqlite::Statement, _col_offset: usize) -> sqlite::Result<(Self, usize)> where Self: Sized {
        unreachable!("sqlite only gives Strings back, not &strs!");
    }
}

impl Modelable for std::string::String {
    fn bind_to(&self, stmt: &mut sqlite::Statement, col: usize) -> sqlite::Result<()> {
        self.as_str().bind(stmt, col)
    }
    fn build_from(stmt: &sqlite::Statement, col_offset: usize) -> sqlite::Result<(Self, usize)> where Self: Sized {
        stmt.read(col_offset).map(|x| (x, 1))
    }
}
